package pcc.chemicraft;

import net.minecraft.block.Block;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.Configuration;
import net.minecraftforge.common.Property;
import net.minecraftforge.oredict.OreDictionary;
import pcc.chemicraft.compounds.CompoundDebug;
import pcc.chemicraft.creativetab.CreativeTabAtomOres;
import pcc.chemicraft.creativetab.CreativeTabChemiCraft;
import pcc.chemicraft.debug.CommandDeleteItem;
import pcc.chemicraft.debug.CommandGenDebugRoom;
import pcc.chemicraft.debug.CommandSetTile;
import pcc.chemicraft.system.CommonProxy;
import pcc.chemicraft.system.PacketHandler;
import pcc.chemicraft.util.Auxiliary;
import pcc.chemicraft.util.Auxiliary.ArrayAuxiliary;
import pcc.chemicraft.util.Auxiliary.MathAuxiliary;
import pcc.chemicraft.util.Auxiliary.NameAuxiliary;
import cpw.mods.fml.common.Mod;
import cpw.mods.fml.common.Mod.Instance;
import cpw.mods.fml.common.Mod.PostInit;
import cpw.mods.fml.common.Mod.PreInit;
import cpw.mods.fml.common.Mod.ServerStarting;
import cpw.mods.fml.common.SidedProxy;
import cpw.mods.fml.common.event.FMLPostInitializationEvent;
import cpw.mods.fml.common.event.FMLPreInitializationEvent;
import cpw.mods.fml.common.event.FMLServerStartingEvent;
import cpw.mods.fml.common.network.NetworkMod;
import cpw.mods.fml.common.network.NetworkRegistry;
import cpw.mods.fml.common.registry.GameRegistry;

/**
 * ChemiCraft本体
 * @author P.C.C.
 */
@Mod(modid = "chemiCraft", name = "ChemiCraft", version = "beta")
@NetworkMod(clientSideRequired = true, serverSideRequired = true, versionBounds = "1.4.6", channels = "chemicraft", packetHandler = PacketHandler.class)
public class ChemiCraft implements Runnable {

	/**
	 * 元素名配列
	 */
	public static final String[] ATOMSNAME = {
		"Hydrogen", "Helium", "Lithium", "Beryllium", "Boron", "Carbon", "Nitrogen", "Oxygen", "Fluorine", "Neon", "Sodium", "Magnesium", "Aluminium", "Silicon",
		"Phosphorus", "Sulfur", "Chlorine", "Argon", "Potassium", "Calcium", "Scandium", "Titanium", "Vanadium", "Chromium", "Manganese", "Iron", "Cobalt", "Nickel", "Copper", "Zinc", "Gallium",
		"Germanium", "Arsenic", "Selenium", "Bromine", "Krypton", "Rubidium", "Strontium", "Yttorium", "Zirconium", "Niobium", "Molybdenum", "Technetium", "Ruthenium", "Rhodium", "Palladium",
		"Silver", "Cadmium", "Indium", "Tin", "Antimony", "Tellurium", "Iodine", "Xenon", "Caesium", "Barium", "Lanthanum", "Cerium", "Praseodymium", "Neodymium", "Promethium", "Samarium",
		"Europium", "Gadolinium", "Terbium", "Dysprosium", "Holmium", "Erbium", "Thulium", "Ytterbium", "Lutetium", "Hafnium", "Tantalum", "Tungsten", "Rhenium", "Osmium", "Iridium", "Platinum",
		"Gold", "Mercury", "Thallium", "Lead", "Bismuth", "Polonium", "Astatine", "Radon", "Francium", "Radium", "Actinium", "Thorium", "Protactinium", "Uranium", "Neptunium", "Plutonium",
		"Americium", "Curium", "Berkelium", "Californium", "Einsteinium", "Fermium", "Mendelevium", "Nobelium", "Lawrencium", "Rutherfordium", "Dubnium", "Seaborgium", "Bohrium", "Hassium",
		"Meitnerium", "Darmstadtium", "Roentgenium", "Copernicium", "Ununtrium", "Ununquadium", "Ununpentium", "Ununhexium", "Ununseptium", "Ununoctium" };

	/**
	 * 元素名日本語版配列
	 */
	public static final String[] ATOMSNAMEJP = {
		"水素", "ヘリウム", "リチウム", "ベリウム", "ホウ素", "炭素", "窒素", "酸素", "フッ素", "ネオン", "ナトリウム", "マグネシウム", "アルミニウム", "ケイ素", "リン", "硫黄", "塩素", "アルゴン", "カリウム", "カルシウム", "スカンジウム",
		"チタン", "バナジウム", "クロム", "マンガン", "鉄", "コバルト", "ニッケル", "銅", "亜鉛", "ガリウム", "ゲルマニウム", "ヒ素", "セレン", "臭素", "クリプトン", "ルビジウム", "ストロンチウム", "イットリウム", "ジルコニウム", "ニオブ", "モリブデン", "テクネチウム", "ルテニウム",
		"ロジウム", "パラジウム", "銀", "カドミウム", "インジウム", "スズ", "アンチモン", "テルル", "ヨウ素", "キセノン", "セシウム", "バリウム", "ランタン", "セリウム", "プラセオジム", "ネオジム", "プロメチウム", "サマリウム", "ユウロビウム", "ガドリニウム", "テルビウム", "ジスプロニウム",
		"ホルミウム", "エルビウム", "ツリウム", "イッテルビウム", "ルテチウム", "ハフニウム", "タンタル", "タングステン", "レニウム", "オスミウム", "イリジウム", "白金", "金", "水銀", "タリウム", "鉛", "ビスマス", "ポロニウム", "アスタチン", "ラドン", "フランシウム", "ラジウム",
		"アクチニウム", "トリウム", "プロトアクチウム", "ウラン", "ネプツニウム", "プルトニウム", "アメリシウム", "キュリウム", "バークリウム", "カルホルニウム", "アインスタイニウム", "フェルミウム", "メンデレビウム", "ノーベリウム", "ローレンシウム", "ラサホージウム", "ドブニウム", "シーボーギウム",
		"ボーリウム", "ハッシウム", "マイトネリウム", "ダームスタチウム", "レントゲニウム", "コペルニシウム", "ウンウントリウム", "ウンウンクアジウム", "ウンウンペンチウム", "ウンウンヘキシウム", "ウンウンセプチウム", "ウンウンオクチウム" };

	/**
	 * this is ChemiCraft instance.
	 */
	@Instance("chemiCraft")
	public static ChemiCraft instance;

	/**
	 * proxy of ChemiCraft.
	 */
	@SidedProxy(clientSide = "pcc.chemicraft.client.ClientProxy", serverSide = "pcc.chemicraft.system.CommonProxy")
	public static CommonProxy proxy;

	/**
	 * CreativeTab of ChemiCraft.
	 */
	public static final CreativeTabs creativeTabChemiCraft = new CreativeTabChemiCraft("ChemiCraft");
	public static final CreativeTabs creativeTabAtomsOre = new CreativeTabAtomOres("AtomsOre");

	/**
	 * the ItemID.
	 */
	public int atomsID;
	public int compoundsID;
	public int gasCollectingBottleID;
	public int atomGrenadeID;
	public int chemicalCellsID;
	public int dustID;
	public int radiationGunID;
	public int radiationBalletID;
	public int blackSmokeID;
	public int pearID;

	/**
	 * the BlockID.
	 */
	public int pyrolysisTableID;
	public int electrolysisTableID;
	public int chemicalConbinationTableID;
	public int toolAndWeaponCraftingTableID;
	public int chemicalCraftingTableID;

	public int[] atomOresID = new int[3];

	/**
	 * the GUIID.
	 */
	public int guiPyrolysisTableID;
	public int guiElectrolysisTableID;
	public int guiChemicalCombinationTableID;
	public int guiToolAndWeaponCraftingTableID;
	public int guiChemicalCraftingTableID;

	/**
	 * Variables of Block type.
	 */
	public Block blockPyrolysisTable;
	public Block blockElectrolysisTable;
	public Block blockChemicalCombinationTable;
	public Block blockToolAndWeaponCraftingTable;
	public Block blockChemicalCraftingTable;

	public Block[] blockAtomOres = new Block[atomOresID.length];

	/**
	 * Variables of Block type.
	 */
	public Item itemAtoms;
	public Item itemCompounds;
	public Item itemGasCollectingBottle;
	public Item itemAtomGrenade;
	public Item itemChemicalCells;
	public Item itemDust;
	public Item itemRadiationGun;
	public Item itemRadiationBallet;
	public Item itemBlackSmoke;
	public Item itemPear;

	/**
	 * this is Textures Path on ChemiCraft.
	 */
	public final String ITEM_ATOMS_TEXTURE = "/pcc/chemicraft/items/Atoms.png";
	public final String ITEM_COMPOUNDS_TEXTURE = "/pcc/chemicraft/items/Compounds.png";
	public final String ITEM_VEGETABLES_TEXTURE = "/pcc/chemicraft/items/Vegetables.png";
	public final String ITEM_TEXTURE = "/pcc/chemicraft/items/items.png";
	public final String BLOCK_CHEMICAL_CRAFTING_TABLE_TEXTURE = "/pcc/chemicraft/blocks/cctable.png";
	public final String BLOCK_ATOMS_ORE_TEXTURE = "/pcc/chemicraft/blocks/AtomsOre.png";
	public final String BLOCK_VEGETABLES_TEXTURE = "/pcc/chemicraft/blocks/Vegetables.png";
	public final String GUI_PYROLYSIS_TEXTURE = "/pcc/chemicraft/guis/Pyrolysis.png";
	public final String GUI_ELECTROLYSIS_TEXTURE = "/pcc/chemicraft/guis/Electrolysis.png";
	public final String GUI_CHEMICALCOMBINATION_TEXTURE = "/pcc/chemicraft/guis/ChemicalCombination.png";
	public final String GUI_TOOLANDWEAPONCRAFTING_TEXTURE = "/pcc/chemicraft/guis/ToolAndWeaponCrafting.png";
	public final String GUI_CHEMICALCRAFTING_TEXTURE = "/pcc/chemicraft/guis/MaterialCrafting.png";
	public final String ENTITY_PARTICLE_TEXRURE = "/pcc/chemicraft/entityParticles/dust.png";

	/**
	 *These Instances required on ChemiCraft.
	 */
	public Auxiliary auxiliary = new Auxiliary();
	public NameAuxiliary nameAuxiliary = new NameAuxiliary();
	public ArrayAuxiliary arrayAuxiliary = new ArrayAuxiliary();
	public MathAuxiliary mathAuxiliary = new MathAuxiliary();

	/**
	 * API Instance and Data Instance.
	 */
	public ChemiCraftAPI api = ChemiCraftAPI.instance();
	public static final ChemiCraftData chemicalData = new ChemiCraftData();

	/**
	 * this is Thread on ChemiCraft.
	 */
	private Thread thread;

	/**
	 * the Event Instance.
	 */
	private FMLPostInitializationEvent event;

	/**
	 * the Register Instances.
	 */
	public ChemiCraftRegisterBlock registerBlock;
	public ChemiCraftRegisterItem registerItem;
	public ChemiCraftRegisterTileEntity registerTileEntity;
	public ChemiCraftRegisterRecipe registerRecipe;
	public ChemiCraftRegisterChemicalRecipe registerChemicalRecipe;
	public ChemiCraftRegisterCompounds registerCompounds;
	public ChemiCraftRegisterCompoundsHandler registerCompoundsHandler;
	public ChemiCraftRegisterEntitys registerEntitys;

	public ChemiCraft() {
		this.registerBlock = new ChemiCraftRegisterBlock(this);
		this.registerItem = new ChemiCraftRegisterItem(this);
		this.registerTileEntity = new ChemiCraftRegisterTileEntity(this);
		this.registerRecipe = new ChemiCraftRegisterRecipe(this);
		this.registerChemicalRecipe = new ChemiCraftRegisterChemicalRecipe(this);
		this.registerCompounds = new ChemiCraftRegisterCompounds(this);
		this.registerCompoundsHandler = new ChemiCraftRegisterCompoundsHandler(this);
		this.registerEntitys = new ChemiCraftRegisterEntitys(this);

		//this.sinVisualizer  = new SinVisualizer();
	}

	/**
	 * PreInit:
	 * Configをロードします。
	 * @param event アノテーション呼び出しにより呼び出す必要なし
	 */
	@PreInit
	public void chemiPreLoadMethod(final FMLPreInitializationEvent event) {
		Configuration cfg = new Configuration(event.getSuggestedConfigurationFile());
		cfg.load();

		Property baseBlockID = cfg.get("BlockID", "Base of Block ID", 2400);
		Property baseItemID = cfg.get("ItemID", "Base of Item ID", 25000);

		ChemiCraftConfiguration ccfgBlock = new ChemiCraftConfiguration(baseBlockID.getInt());
		ChemiCraftConfiguration ccfgItem = new ChemiCraftConfiguration(baseBlockID.getInt());

		this.pyrolysisTableID = ccfgBlock.additionID();
		this.electrolysisTableID = ccfgBlock.additionID();
		this.chemicalConbinationTableID = ccfgBlock.additionID();
		this.toolAndWeaponCraftingTableID = ccfgBlock.additionID();
		this.chemicalCraftingTableID = ccfgBlock.additionID();
		for (int i = 0; i < atomOresID.length; i++) {
			this.atomOresID[i] = ccfgBlock.additionID();
		}

		this.atomsID = ccfgItem.additionID();
		this.compoundsID = ccfgItem.additionID();
		this.gasCollectingBottleID = ccfgItem.additionID();
		this.atomGrenadeID = ccfgItem.additionID();
		this.chemicalCellsID = ccfgItem.additionID();
		this.dustID = ccfgItem.additionID();
		this.radiationGunID = ccfgItem.additionID();
		this.radiationBalletID = ccfgItem.additionID();
		this.blackSmokeID = ccfgItem.additionID();

		Property guiPyrolysisTableIDProp = cfg.get("GUI", "GUIPyrolysisID", 1000);
		Property guiElectrolysisTableIDProp = cfg.get("GUI", "GUIElectrolysisTableIDProp", 1001);
		Property guiChemicalCombinationTableIDProp = cfg.get("GUI", "GUIChemicalCombinationTableID", 1002);
		Property guiToolAndWeaponCraftingTableIDProp = cfg.get("GUI", "GUIToolAndWeaponCraftingTableID", 1003);
		Property guiChemicalCraftingTableIDProp = cfg.get("GUI", "GUIChemicalCraftingTableID", 1004);

		this.guiPyrolysisTableID = guiPyrolysisTableIDProp.getInt();
		this.guiElectrolysisTableID = guiElectrolysisTableIDProp.getInt();
		this.guiChemicalCombinationTableID = guiChemicalCombinationTableIDProp.getInt();
		this.guiToolAndWeaponCraftingTableID = guiToolAndWeaponCraftingTableIDProp.getInt();
		this.guiChemicalCraftingTableID = guiChemicalCraftingTableIDProp.getInt();

		/*
		Property pyrolysisTableIDProp = cfg.getBlock("PyrolysisTable", 2400);
		Property electrolysisTableIDProp = cfg.getBlock("ElectrolysisTable", 2401);
		Property chemicalCombinationTableIDProp = cfg.getBlock("ChemicalCombinationID", 2402);
		Property toolAndWeaponCraftingTableIDProp = cfg.getBlock("ToolAndWeaponCraftingTableID", 2403);
		Property chemicalCraftingTableIDProp = cfg.getBlock("ChemicalCraftingTableID", 2404);

		Property[] atomOresIDProp = new Property[this.atomOresID.length];
		for (int i = 0; i < atomOresIDProp.length; i++) {
			atomOresIDProp[i] = cfg.getBlock("AtomsOreID:" + i, 2500 + i);
		}

		Property atomsIDProp = cfg.getItem("AtomsID", 25000);
		Property compoundsIDProp = cfg.getItem("CompoundsID", 25001);
		Property gasCollectingBottleIDProp = cfg.getItem("gasCollectingBottleID", 25002);
		Property atomGrenadeIDProp = cfg.getItem("AtomGrenadeID", 25003);
		Property chemicalCellsIDProp = cfg.getItem("ChemicalCellID", 25004);
		Property dustIDProp = cfg.getItem("DustID", 25005);
		Property radiationGunIDProp = cfg.getItem("RadiationGunID", 25006);
		Property radiationBalletIDProp = cfg.getItem("RadiationBalletID", 25007);

		Property guiPyrolysisTableIDProp = cfg.get("GUI", "GUIPyrolysisID", 1000);
		Property guiElectrolysisTableIDProp = cfg.get("GUI", "GUIElectrolysisTableIDProp", 1001);
		Property guiChemicalCombinationTableIDProp = cfg.get("GUI", "GUIChemicalCombinationTableID", 1002);
		Property guiToolAndWeaponCraftingTableIDProp = cfg.get("GUI", "GUIToolAndWeaponCraftingTableID", 1003);
		Property guiChemicalCraftingTableIDProp = cfg.get("GUI", "GUIChemicalCraftingTableID", 1004);


		this.pyrolysisTableID = pyrolysisTableIDProp.getInt();
		this.electrolysisTableID = electrolysisTableIDProp.getInt();
		this.chemicalConbinationTableID = chemicalCombinationTableIDProp.getInt();
		this.toolAndWeaponCraftingTableID = toolAndWeaponCraftingTableIDProp.getInt();
		this.chemicalCraftingTableID = chemicalCraftingTableIDProp.getInt();
		for (int i = 0; i < atomOresID.length; i++) {
			this.atomOresID[i] = atomOresIDProp[i].getInt();
		}

		this.atomsID = atomsIDProp.getInt();
		this.compoundsID = compoundsIDProp.getInt();
		this.gasCollectingBottleID = gasCollectingBottleIDProp.getInt();
		this.atomGrenadeID = atomGrenadeIDProp.getInt();
		this.chemicalCellsID = chemicalCellsIDProp.getInt();
		this.dustID = dustIDProp.getInt();
		this.radiationGunID = radiationGunIDProp.getInt();
		this.radiationBalletID = radiationBalletIDProp.getInt();

		this.guiPyrolysisTableID = guiPyrolysisTableIDProp.getInt();
		this.guiElectrolysisTableID = guiElectrolysisTableIDProp.getInt();
		this.guiChemicalCombinationTableID = guiChemicalCombinationTableIDProp.getInt();
		this.guiToolAndWeaponCraftingTableID = guiToolAndWeaponCraftingTableIDProp.getInt();
		this.guiChemicalCraftingTableID = guiChemicalCraftingTableIDProp.getInt();
		*/

		cfg.save();
	}



	@ServerStarting
	public void serverStarting(final FMLServerStartingEvent event) {
		event.registerServerCommand(new CommandSetTile());
		event.registerServerCommand(new CommandDeleteItem());
		event.registerServerCommand(new CommandGenDebugRoom());
	}



	@PostInit
	public void chemiPostLoadMethod(final FMLPostInitializationEvent event) {
		this.thread = new Thread(this);
		this.event = event;
		this.thread.start();

		try {
			this.thread.join();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		proxy.registerRenderInformation();
		Thread.yield();
	}



	@Override
	public void run() {
		while (true) {
			if (proxy != null && instance != null) {
				this.settingProcessing(event);
				this.apiProcessing(event);
				this.debug(event);
				break;
			}
		}
	}



	private void settingProcessing(final FMLPostInitializationEvent event) {

		this.registerBlock.start();
		this.registerItem.start();
		this.registerTileEntity.start();
		this.registerRecipe.start();
		this.registerChemicalRecipe.start();
		this.registerCompounds.start();
		this.registerCompoundsHandler.start();
		this.registerEntitys.start();

		// GUIを追加します
		NetworkRegistry.instance().registerGuiHandler(instance, proxy);

		//電池を追加します
		this.api.addChemicalCell(itemChemicalCells, "ChemicalCell", 1000);
		this.api.addChemicalCellLanguage("ChemicalCell", "ja_JP", "化学電池");

		this.api.addPyrolysisDecompositionFuel(new ItemStack(Item.coal), 2000*8);
	}



	private void apiProcessing(final FMLPostInitializationEvent event) {
		// API用の処理
		for (int i = 0; i < api.getAtomOresName().sizeKeysList(); i++)
		{
			for (int j = 0; j < api.getAtomOresName().sizeElementsList(api.getAtomOresName().getKeyList(i)); j++)
			{
				this.nameAuxiliary.addName(new ItemStack(blockAtomOres[i / 16], 0, i - i / 16 * 16), api.getAtomOresLanguage().get(api.getAtomOresName().getKeyList(i), j), api.getAtomOresName().get(api.getAtomOresName().getKeyList(i), j));
				OreDictionary.registerOre("ore" + api.getAtomOresName().getKeyList(i), new ItemStack(blockAtomOres[i / 16], 1, i - i / 16 * 16));
			}
		}

		for (int i = 0; i < api.getAtomOres().size(); i++)
		{
			GameRegistry.registerWorldGenerator(api.getAtomOres().get(i));
		}

		for (int i = 0; i < api.getChemicalCellsName().sizeKeysList(); i++)
		{
			for (int j = 0; j < api.getChemicalCellsName().sizeElementsList(api.getChemicalCellsName().getKeyList(i)); j++)
			{
				this.nameAuxiliary.addName(new ItemStack(itemChemicalCells, 0, i), api.getChemicalCellsLanguage().get(api.getChemicalCellsName().getKeyList(i), j), api.getChemicalCellsName().get(api.getChemicalCellsName().getKeyList(i), j));
			}
		}

		for (int i = 0; i < api.getCompoundsName().sizeKeysList(); i++)
		{
			for (int j = 0; j < api.getCompoundsName().sizeElementsList(api.getCompoundsName().getKeyList(i)); j++)
			{
				this.nameAuxiliary.addName(new ItemStack(itemCompounds, 0, i), api.getCompoundsLang().get(api.getCompoundsName().getKeyList(i), j), api.getCompoundsName().get(api.getCompoundsName().getKeyList(i), j));
			}
		}

	}



	private void debug(final FMLPostInitializationEvent event) {
		this.proxy.registerTickHandler();
		this.api.addCompound("Debug");
		this.api.settingCompoundHandler("Debug", new CompoundDebug());

		this.api.addToolAndWeaponRecipe(new ItemStack[] {
				new ItemStack(Block.stone, 1, 1)
		},
		new ItemStack(Block.dirt, 1, 0));

	}

}
